@@ -13,6 +13,7 @@
 #include <sys/stat.h>
 #include <utime.h>
 #include <stdbool.h>
+#include <signal.h>
 
 #include "gif2png.h"
 
@@ -123,18 +124,15 @@ static int writefile(struct GIFelement *s, struct GIFelement *e,
     GifColor *colors = img->colors;
     int colors_used = 0;
     byte remap[MAXCMSIZE];
-    bool low_prec;
     png_struct *png_ptr;
     png_info *info_ptr;
     int p;
-    int gray_bitdepth;
-    png_color pal_rgb[MAXCMSIZE], *pltep;
+    png_color pal_rgb[MAXCMSIZE];
     png_byte pal_trans[MAXCMSIZE];
     png_color_16 color16trans, color16back;
     byte buffer[24]; /* used for gIFt and gIFg */
     int j;
     png_uint_16 histogr[MAXCMSIZE];
-    unsigned long hist_maxvalue;
     int passcount;
     int errtype, errorcount = 0;
     png_text software;
@@ -230,6 +228,8 @@ static int writefile(struct GIFelement *s, struct GIFelement *e,
     if (last_color<2) bitdepth = 1;
 
     if (gray) {
+	bool low_prec;
+	int gray_bitdepth;
 	remapping = true;
 	for (i = 0; i < MAXCMSIZE; i++)
 	    remap[i] = colors[i].red;
@@ -433,6 +433,7 @@ static int writefile(struct GIFelement *s, struct GIFelement *e,
 
     /* generate the PNG palette */
     if (!gray) {
+	png_color *pltep;
 	if (remapping) {
 	    if (verbose > 2)
 		(void)fprintf(stderr, "Remapping palette...\n");
@@ -462,7 +463,7 @@ static int writefile(struct GIFelement *s, struct GIFelement *e,
 
     /* generate histogram chunk for paletted images only */
     if (histogram && !gray) {
-	hist_maxvalue = 0;
+	unsigned long hist_maxvalue = 0;
 	for (i = 0; i < MAXCMSIZE; i++)
 	    if (count[i]>hist_maxvalue)
 		hist_maxvalue = count[i];
@@ -818,15 +819,17 @@ static int processfile(char *fname, FILE *fp)
 }
 /*@=onlytrans@*/
 
-/* check if stdin is a terminal, if your compiler is lacking isatty or fileno
-   (non-ANSI functions), just return false
-*/
-
 static bool input_is_terminal(void)
 {
     return isatty(fileno(stdin))!=0;
 }
 
+static void bailout(int sig)
+{
+    (void)fprintf(stderr, "gif2png: GIF is fatally malformed, bailing out.\n");
+    exit(2);
+}
+
 int main(int argc, char *argv[])
 {
     FILE *fp;
@@ -837,6 +840,8 @@ int main(int argc, char *argv[])
     int ac;
     char *color;
 
+    signal(SIGSEGV, bailout);
+
     software_chunk = true;
 
     for (ac = 1; ac < argc && argv[ac][0] == '-'; ac++)
@@ -853,7 +858,7 @@ int main(int argc, char *argv[])
 		color = argv[++ac];
 		if (*color == '#')
 		    color++;
-	        (void)sscanf(color, "%ux", &background);
+	        (void)sscanf(color, "%2ux", &background);
 		matte_color.red = (background >> 16) & 0xff;
 		matte_color.green = (background >> 8) & 0xff; 
 		matte_color.blue = background & 0xff;
@@ -919,9 +924,9 @@ int main(int argc, char *argv[])
 	    usage:
 		(void)fprintf(stderr,
 	"\n"
-	"usage: gif2png [ -bdfghinprstvwO ] [file[.gif]] ...\n"
+	"usage: gif2png [ -bdfghimnprstvwO ] [file[.gif]] ...\n"
 	"\n"
-	"   -b  replace background pels with given RRGGBB color (hex digits)\n"
+	"   -b  replace background pixels with given RRGGBB color (hex digits)\n"
 	"   -d  delete source GIF files after successful conversion\n"
 	"   -f  allow use as filter, die on multi-image GIF\n"
 	"   -g  write gamma=(1.0/2.2) and sRGB chunks\n"
@@ -932,9 +937,8 @@ int main(int argc, char *argv[])
 	"   -p  display progress of PNG writing in %%\n"
 	"   -r  try to recover corrupted GIF files\n"
 	"   -s  do not write Software text chunk\n"
-	"   -t  transparency OK, web probe mode accepts transparent GIFs\n"
 	"   -v  verbose; display conversion statistics and debugging messages\n"
-	"   -w  web probe, list images without animation or transparency\n"
+	"   -w  web probe, list images without animation\n"
 	"   -O  optimize; remove unused color-table entries; use zlib level 9\n"
 	"   -1  one-only; same as -f; provided for backward compatibility\n"
 	"\n"
@@ -996,5 +1000,5 @@ int main(int argc, char *argv[])
 		      errors!=0? "with one or more errors" : "no errors detected", 
 		      numgifs, (numgifs == 1) ? "" : "s", numpngs, (numpngs == 1)? "" : "s");
 
-    return errors;
+    return (errors > 0) ? 1 : 0;
 }
