@@ -73,7 +73,6 @@ ReadGIF(FILE *fd)
     bool          useGlobalColormap;
     unsigned int  bitPixel;
     char          gif_version[4];
-    int i;
     int w, h, x_off, y_off;
 
     imagecount=0;
@@ -132,6 +131,7 @@ ReadGIF(FILE *fd)
 	 */
 
 	static int colors[]={0, 7, 1, 2, 4, 3, 5, 6};
+	int i;
 
 	/*@-predboolint -type@*/
 	for(i=0 ; i<MAXCMSIZE-8 ; i++) {
@@ -509,7 +509,12 @@ static int nextCode(FILE *fd, int code_size)
 	ret = (long)buf[i] | ((long)buf[i+1] << 8) | ((long)buf[i+2] << 16);
 
     /*@-shiftnegative@*/
-    ret = (ret >> (curbit % 8)) & maskTbl[code_size];
+    if( code_size < 16) {
+        ret = (ret >> (curbit % 8)) & maskTbl[code_size];
+    } else {
+        (void)fprintf(stderr, "gif2png: LWZMinimumCodeSize can't be bigger than 15\n");
+        exit(1);
+    }
     /*@=shiftnegative@*/
     /*@=shiftimplementation@*/
 
@@ -524,12 +529,12 @@ static int nextLWZ(FILE *fd)
 {
     static int       table[2][(1<< MAX_LWZ_BITS)];
     static int       firstcode, oldcode;
-    int              code, incode;
+    int              code;
     register int     i;
 
     while ((code = nextCode(fd, code_size)) >= 0) {
+	int              incode;
 	if (code == clear_code) {
-
 	    /* corrupt GIFs can make this happen */
 	    if (clear_code >= (1<<MAX_LWZ_BITS)) {
 		return -2;
@@ -578,11 +583,11 @@ static int nextLWZ(FILE *fd)
 	    *sp++ = table[1][code];
 	    if (code == table[0][code]) {
 		(void)fprintf(stderr, "gif2png: circular table entry BIG ERROR\n");
-		return(code);
+		exit(1);
 	    }
 	    if ((size_t)((char *)sp - (char *)stack) >= sizeof(stack)) {
 		(void)fprintf(stderr, "gif2png: circular table STACK OVERFLOW!\n");
-		return(code);
+		exit(1);
 	    }
 	    code = table[0][code];
 	}
