@@ -111,13 +111,13 @@ void fix_current(void)
 /*@-nullstate@*/
 void free_mem(void)
 {
-    struct GIFelement *p,*t;
+    struct GIFelement *p;
 
     p=first.next;
     first.next=NULL;
 
     while(p) {
-	t=p;
+	struct GIFelement *t=p;
 	p=p->next;
 	if(t->data) free(t->data);
 	if(t->imagestruct) free(t->imagestruct);
